@@ -20,7 +20,7 @@ come with the distribution.
 
 =cut
 
-$VERSION = '0.96';
+$VERSION = '0.97';
 use strict;
 use B;
 use Sepia::Debug;               # THIS TURNS ON DEBUGGING INFORMATION!
@@ -29,7 +29,7 @@ use Scalar::Util 'looks_like_number';
 use Text::Abbrev;
 
 use vars qw($PS1 %REPL %RK %REPL_DOC %REPL_SHORT %PRINTER
-            @REPL_RESULT
+            @REPL_RESULT @res
             $REPL_LEVEL $PACKAGE $WANTARRAY $PRINTER $STRICT $PRINT_PRETTY
             $ISEVAL);
 
@@ -82,25 +82,31 @@ sub repl_size
         print "Size requires Devel::Size.\n";
     } else {
         *Sepia::repl_size = sub {
+            no strict 'refs';
             ## XXX: C&P from repl_who:
             my ($pkg, $re) = split ' ', shift || '';
             if ($pkg =~ /^\/(.*)\/?$/) {
                 $pkg = $PACKAGE;
                 $re = $1;
-            } elsif (!$re && !defined %{$pkg.'::'}) {
+            } elsif (!$re && !%{$pkg.'::'}) {
                 $re = $pkg;
                 $pkg = $PACKAGE;
+            } else {
+                $re = '';
+                $pkg = $PACKAGE;
             }
             my @who = who($pkg, $re);
-            my $len = max(map { length } @who) + 4;
+            my $len = max(3, map { length } @who) + 4;
             my $fmt = '%-'.$len."s%10d\n";
             print 'Var', ' ' x ($len + 2), "Bytes\n";
             print '-' x ($len-4), ' ' x 9, '-' x 5, "\n";
             local $SIG{__WARN__} = sub {};
             for (@who) {
-                my $res = eval "package $pkg; Devel::Size::total_size \\$_;";
-                next if $res == 0;
-                printf $fmt, $_, $res || 0;
+                next unless /^[\$\@\%\&]/; # skip subs.
+                # print STDERR "no strict; package $pkg; Devel::Size::total_size \\$_;";
+                my $res = eval "no strict; package $pkg; Devel::Size::total_size \\$_;";
+                print "aiee: $@\n" if $@;
+                printf $fmt, $_, $res;
             }
         };
         goto &repl_size;
@@ -166,7 +172,7 @@ sub filter_untyped
 {
     no strict;
     local $_ = /^::/ ? $_ : "::$_";
-    defined *{$_}{CODE} || defined *{$_}{IO} || (/::$/ && defined *{$_}{HASH});
+    defined *{$_}{CODE} || defined *{$_}{IO} || (/::$/ && %$_);
 }
 
 ## XXX: Careful about autovivification here!  Specifically:
@@ -179,9 +185,9 @@ sub filter_typed
     my $type = shift;
     local $_ = /^::/ ? $_ : "::$_";
     if ($type eq 'SCALAR') {
-        defined ${$_};
+        defined $$_;
     } elsif ($type eq 'VARIABLE') {
-        defined ${$_} || defined *{$_}{HASH} || defined *{$_}{ARRAY};
+        defined $$_ || defined *{$_}{HASH} || defined *{$_}{ARRAY};
     } else {
         defined *{$_}{$type}
     }
@@ -244,10 +250,19 @@ sub all_completions
 
 sub completions
 {
-    my ($type, $str) = $_[0] =~ /^([\%\$\@\&]?)(.*)/;
+    my ($type, $str, $t);
     my %h = qw(@ ARRAY % HASH & CODE * IO $ SCALAR);
-    my $t = $type || '';
+    my %rh;
+    @rh{values %h} = keys %h;
+    if (@_ == 1) {
+        ($type, $str) = $_[0] =~ /^([\%\$\@\&]?)(.*)/;
+        $t = $type || '';
     $type = $h{$type} if $type;
+    } else {
+        ($str, $type) = @_;
+        $type ||= '';
+        $t = $rh{$type} if $type;
+    }
     my @ret = grep {
         $type ? filter_typed $type : filter_untyped
     } all_completions $str;
@@ -272,7 +287,7 @@ sub method_completions
     $eval ||= 'CORE::eval';
     no strict;
     return unless ($x =~ /^\$/ && ($x = $eval->("ref($x)")))
-        || $eval->('defined(%{'.$x.'::})');
+        || $eval->('%'.$x.'::');
     unless ($@) {
         my $re = _apropos_re $fn;
         ## Filter out overload methods "(..."
@@ -361,7 +376,7 @@ sub apropos
     no strict;
     if ($it =~ /^(.*::)([^:]+)$/) {
         my ($stash, $name) = ($1, $2);
-        if (!defined %$stash) {
+        if (!%$stash) {
             return;
         }
         if ($re) {
@@ -373,7 +388,9 @@ sub apropos
                 my $stashnm = "$stash$_";
                 /$name/ &&
                     (($stashp && /::$/)
-                     || scalar grep { defined *{$stashnm}{$_} } @types)
+                     || scalar grep {
+                         defined($_ eq 'SCALAR' ? $$stashnm : *{$stashnm}{$_})
+                     } @types)
             } keys %$stash;
         } else {
             defined &$it ? $it : ();
@@ -399,7 +416,7 @@ sub mod_subs
     no strict;
     my $p = shift;
     my $stash = \%{"$p\::"};
-    if (defined $stash) {
+    if (%$stash) {
         grep { defined &{"$p\::$_"} } keys %$stash;
     }
 }
@@ -603,7 +620,6 @@ which can use either L<Data::Dumper>, L<YAML>, or L<Data::Dump>.
         local $Data::Dumper::Deparse = 1;
         local $Data::Dumper::Indent = 0;
         local $_;
-        no strict;
         my $thing = @res > 1 ? \@res : $res[0];
         eval {
             $_ = Data::Dumper::Dumper($thing);
@@ -623,11 +639,9 @@ which can use either L<Data::Dumper>, L<YAML>, or L<Data::Dump>.
         $_;
     },
     plain => sub {
-        no strict;
         "@res";
     },
     yaml => sub {
-        no strict;
         eval { require YAML };
         if ($@) {
             $PRINTER{dumper}->();
@@ -636,7 +650,6 @@ which can use either L<Data::Dumper>, L<YAML>, or L<Data::Dump>.
         }
     },
     dump => sub {
-        no strict;
         eval { require Data::Dump };
         if ($@) {
             $PRINTER{dumper}->();
@@ -648,9 +661,9 @@ which can use either L<Data::Dumper>, L<YAML>, or L<Data::Dump>.
 
 sub printer
 {
-    no strict;
     local *res = shift;
     my ($wantarray) = @_;
+    my $res;
     @::__ = @res;
     $::__ = @res == 1 ? $res[0] : [@res];
     my $str;
@@ -674,7 +687,6 @@ sub printer
 }
 
 BEGIN {
-    no strict;
     $PS1 = "> ";
     $PACKAGE = 'main';
     $WANTARRAY = 1;
@@ -697,7 +709,7 @@ sub Dump
 sub flow
 {
     my $n = shift;
-    my $n1 = int($n/2);
+    my $n1 = int(2*$n/3);
     local $_ = shift;
     s/(.{$n1,$n}) /$1\n/g;
     $_
@@ -868,17 +880,31 @@ sub repl_pwd
 
 sub who
 {
-    my ($pack, $re) = @_;
-    $re ||= '.?';
-    $re = qr/$re/;
+    my ($pack, $re_str) = @_;
+    $re_str ||= '.?';
+    my $re = qr/$re_str/;
     no strict;
-    sort grep /$re/, map {
-        (defined %{$pack.'::'.$_} ? '%'.$_ : (),
-         defined ${$pack.'::'.$_} ? '$'.$_ : (), # ?
-         defined @{$pack.'::'.$_} ? '@'.$_ : (),
-         defined &{$pack.'::'.$_} ? $_ : (),
-     )
-    } grep !/::$/ && !/^(?:_<|[^\w])/, keys %{$pack.'::'};
+    if ($re_str =~ /^[\$\@\%\&]/) {
+        ## sigil given -- match it
+        sort grep /$re/, map {
+            my $name = $pack.'::'.$_;
+            (defined *{$name}{HASH} ? '%'.$_ : (),
+             defined *{$name}{ARRAY} ? '@'.$_ : (),
+             defined *{$name}{CODE} ? $_ : (),
+             defined ${$name} ? '$'.$_ : (), # ?
+         )
+        } grep !/::$/ && !/^(?:_<|[^\w])/ && /$re/, keys %{$pack.'::'};
+    } else {
+        ## no sigil -- don't match it
+        sort map {
+            my $name = $pack.'::'.$_;
+            (defined *{$name}{HASH} ? '%'.$_ : (),
+             defined *{$name}{ARRAY} ? '@'.$_ : (),
+             defined *{$name}{CODE} ? $_ : (),
+             defined ${$name} ? '$'.$_ : (), # ?
+         )
+        } grep !/::$/ && !/^(?:_<|[^\w])/ && /$re/, keys %{$pack.'::'};
+    }
 }
 
 
@@ -895,7 +921,8 @@ sub columnate
     my @incs = map { $_ * $nr } 0..$nc-1;
     my $str = '';
     for my $r (0..$nr-1) {
-        $str .= sprintf $fmt, map { $_ || '' } @_[map { $r + $_ } @incs];
+        $str .= sprintf $fmt, map { defined($_) ? $_ : '' }
+            @_[map { $r + $_ } @incs];
     }
     $str =~ s/ +$//m;
     $str
@@ -904,10 +931,11 @@ sub columnate
 sub repl_who
 {
     my ($pkg, $re) = split ' ', shift;
+    no strict;
     if ($pkg =~ /^\/(.*)\/?$/) {
         $pkg = $PACKAGE;
         $re = $1;
-    } elsif (!$re && !defined %{$pkg.'::'}) {
+    } elsif (!$re && !%{$pkg.'::'}) {
         $re = $pkg;
         $pkg = $PACKAGE;
     }
@@ -924,7 +952,7 @@ sub methods
         : grep {
             defined *{"$pack\::$_"}{CODE}
         } keys %{$pack.'::'};
-    (@own, defined @{$pack.'::ISA'}
+    (@own, defined *{$pack.'::ISA'}{ARRAY}
          ? (map methods($_, $qualified), @{$pack.'::ISA'}) : ());
 }
 
@@ -958,7 +986,7 @@ sub repl_package
 {
     chomp(my $p = shift);
     no strict;
-    if (defined %{$p.'::'}) {
+    if (%{$p.'::'}) {
         $PACKAGE = $p;
 #         my $ecmd = '(setq sepia-eval-package "'.$p.'")';
 #         print ";;;###".length($ecmd)."\n$ecmd\n";
@@ -1054,11 +1082,15 @@ Execute a command interpreter on standard input and standard output.
 If you want to use different descriptors, localize them before
 calling C<repl()>.  The prompt has a few bells and whistles, including:
 
-  * Obviously-incomplete lines are treated as multiline input (press
-    'return' twice or 'C-c' to discard).
+=over 4
 
-  * C<die> is overridden to enter a debugging repl at the point
-    C<die> is called.
+=item Obviously-incomplete lines are treated as multiline input (press
+'return' twice or 'C-c' to discard).
+
+=item C<die> is overridden to enter a debugging repl at the point
+C<die> is called.
+
+=back
 
 Behavior is controlled in part through the following package-globals:
 
@@ -1187,7 +1219,12 @@ sub repl
                     } else {
                         print_warnings;
                         # $@ =~ s/(.*) at eval .*/$1/;
-                        print "error: $@\n";
+                        # don't complain if we're abandoning execution
+                        # from the debugger.
+                        unless (ref $@ eq 'Sepia::Debug') {
+                            print "error: $@";
+                            print "\n" unless $@ =~ /\n\z/;
+                        }
                         print prompt;
                         $buf = '';
                     }
@@ -1211,16 +1248,18 @@ sub perl_eval
     tolisp($REPL{eval}->(shift));
 }
 
-=head2 C<$status = html_module_list($file [, $prefix])>
+=head2 C<$status = html_module_list([$file [, $prefix]])>
 
 Generate an HTML list of installed modules, looking inside of
-packages.  If C<$prefix> is missing, uses "about://perldoc/".
+packages.  If C<$prefix> is missing, uses "about://perldoc/".  If
+$file is given, write the result to $file; otherwise, return it as a
+string.
 
-=head2 C<$status = html_package_list($file [, $prefix])>
+=head2 C<$status = html_package_list([$file [, $prefix]])>
 
 Generate an HTML list of installed top-level modules, without looking
 inside of packages.  If C<$prefix> is missing, uses
-"about://perldoc/".
+"about://perldoc/".  $file is the same as for C<html_module_list>.
 
 =cut
 
@@ -1230,7 +1269,8 @@ sub html_module_list
     $base ||= 'about://perldoc/';
     my $inst = inst();
     return unless $inst;
-    return unless open OUT, ">$file";
+    my $out;
+    open OUT, ">", $file || \$out or return;
     print OUT "<html><body><ul>";
     my $pfx = '';
     my %ns;
@@ -1241,8 +1281,8 @@ sub html_module_list
         print OUT qq{<li><b>$_</b><ul>} if @{$ns{$_}} > 1;
         for (sort @{$ns{$_}}) {
             my @fs = map {
-                s/.*man.\///; s|/|::|g; s/\..?pm//; $_
-            } grep /\.\dpm$/, sort $inst->files($_);
+                s/.*man.\///; s|/|::|g; s/\.\d(?:pm)?$//; $_
+            } grep /\.\d(?:pm)?$/, sort $inst->files($_);
             if (@fs == 1) {
                 print OUT qq{<li><a href="$base$fs[0]">$fs[0]</a>};
             } else {
@@ -1257,7 +1297,7 @@ sub html_module_list
     }
     print OUT "</ul></body></html>\n";
     close OUT;
-    1;
+    $file ? 1 : $out;
 }
 
 sub html_package_list
@@ -1265,7 +1305,8 @@ sub html_package_list
     my ($file, $base) = @_;
     return unless inst();
     $base ||= 'about://perldoc/';
-    return unless open OUT, ">$file";
+    my $out;
+    open OUT, ">", $file || \$out or return;
     print OUT "<html><body><ul>";
     my $pfx = '';
     my %ns;
@@ -1285,7 +1326,7 @@ sub html_package_list
     }
     print OUT "</ul></body></html>\n";
     close OUT;
-    1;
+    $file ? 1 : $out;
 }
 
 1;
@@ -1295,6 +1336,14 @@ __END__
 
 See the README file included with the distribution.
 
+=head1 SEE ALSO
+
+Sepia's public GIT repository is located at L<http://repo.or.cz/w/sepia.git>.
+
+There are several modules for Perl development in Emacs on CPAN,
+including L<Devel::PerlySense> and L<PDE>.  For a complete list, see
+L<http://emacswiki.org/cgi-bin/wiki/PerlLanguage>.
+
 =head1 AUTHOR
 
 Sean O'Rourke, E<lt>seano@cpan.orgE<gt>
@@ -1303,7 +1352,7 @@ Bug reports welcome, patches even more welcome.
 
 =head1 COPYRIGHT
 
-Copyright (C) 2005-2007 Sean O'Rourke.  All rights reserved, some
+Copyright (C) 2005-2008 Sean O'Rourke.  All rights reserved, some
 wrongs reversed.  This module is distributed under the same terms as
 Perl itself.
 
