@@ -1,5 +1,6 @@
 package Sepia::CPAN;
 use CPAN ();
+use LWP::Simple;
 
 sub init
 {
@@ -11,36 +12,7 @@ sub init
 sub interesting_parts
 {
     my $mod = shift;
-    # XXX: stupid CPAN.pm functions die for some modules...
-    +{ map {
-        $_ => scalar eval { $mod->$_ }
-    } qw(id cpan_version inst_version fullname cpan_file)};
-}
-
-# Only list the "root" module of each package, meaning either (1) the
-# module matching the dist name or (2) the module with the shortest
-# name, whichever comes first.
-
-# XXX: this is hacky.
-sub group_by_dist
-{
-    my %h;
-    for (@_) {
-        my $cf = $_->{cpan_file};
-        if (!exists $h{$cf}) {
-            $h{$_->{cpan_file}} = $_;
-        } else {
-            (my $tmp = $cf) =~ s/-/::/g;
-            if ($tmp =~ /^\Q$h{$cf}{id}\E/) {
-                next;           # already perfect
-            } elsif ($tmp =~ /^\Q$_->{id}\E/) {
-                $h{$cf} = $_;   # perfect
-            } # elsif (length $h{$cf}{id} > length $_->{id}) {
-            #     $h{$cf} = $_;   # short, at least...
-            # }
-        }
-    }
-    sort { $a->{id} cmp $b->{id} } values %h;
+    +{ map { $_ => scalar $mod->$_ } qw(id cpan_version inst_version fullname cpan_file)};
 }
 
 sub _list
@@ -50,7 +22,7 @@ sub _list
 
 sub list
 {
-    group_by_dist map { interesting_parts $_ } _list @_
+    map { interesting_parts $_ } _list @_
 }
 
 sub _ls
@@ -64,21 +36,20 @@ sub _ls
 
 sub ls
 {
-    group_by_dist map { interesting_parts $_ } _ls @_
+    map { interesting_parts $_ } _ls @_
 }
 
 sub _desc
 {
     my $pat = qr/$_[0]/i;
     grep {
-        $_->description &&
         ($_->description =~ /$pat/ || $_->id =~ /$pat/)
     } CPAN::Shell->expand('Module', '/./');
 }
 
 sub desc
 {
-    group_by_dist map { interesting_parts $_ } _desc @_;
+    map { interesting_parts $_ } _desc @_;
 }
 
 sub outdated
@@ -114,14 +85,7 @@ sub readme
 
 sub perldoc
 {
-    eval q{ use LWP::Simple; };
-    if ($@) {
-        print STDERR "Can't get perldocs: LWP::Simple not installed.\n";
-        "Can't get perldocs: LWP::Simple not installed.\n";
-    } else {
-        *perldoc = sub { get($CPAN::Defaultdocs . shift) };
-        goto &perldoc;
-    }
+    get($CPAN::Defaultdocs . shift);
 }
 
 sub install
@@ -153,7 +117,7 @@ sub _recommend
 
 sub recommend
 {
-    group_by_dist map { interesting_parts $_ } _recommend @_;
+    map { interesting_parts $_ } _recommend @_;
 }
 
 1;
