@@ -12,13 +12,19 @@ sub define_shortcut;
 BEGIN {
     ## Just leave it on -- with $DB::trace = 0, there doesn't seem
     ## to be a perforamnce penalty!
-    $^P = 0x303;
+    $^P = 0x313;                # 01 | 02 | 10 | 100 | 200
     $STOPDIE = 1;
     $STOPWARN = 0;
+}
 
-    eval { require PadWalker; import PadWalker qw(peek_my) };
+sub peek_my
+{
+    eval { require PadWalker };
     if ($@) {
-        *peek_my = sub { +{ } };
+        +{ }
+    } else {
+        *peek_my = \&PadWalker::peek_my;
+        goto &peek_my;
     }
 }
 
@@ -33,7 +39,8 @@ sub repl_backtrace
     for (my $i = 0; ; ++$i) {
         my ($pack, $file, $line, $sub) = caller($i);
         last unless $pack;
-        print($i == $level+3 ? "*" : ' ', " [$i]\t$sub ($file:$line)\n");
+        # XXX: 4 is the magic number...
+        print($i == $level+4 ? "*" : ' ', " [$i]\t$sub ($file:$line)\n");
     }
 }
 
@@ -48,6 +55,45 @@ sub repl_return
     last repl;
 }
 
+use vars qw($DIE_TO @DIE_RETURN $DIE_LEVEL);
+$DIE_LEVEL = 0;
+
+sub repl_xreturn
+{
+    ($DB::DIE_TO, $DB::DIE_RETURN[0]) = split ' ', $_[0], 2;
+    $DB::DIE_RETURN[0] = $Sepia::REPL{eval}->($DB::DIE_RETURN[0]);
+    last SEPIA_DB_SUB;
+}
+
+# { package DB;
+#  no strict;
+sub sub
+{
+    no strict;
+    local $DIE_LEVEL = $DIE_LEVEL + 1;
+    ## Set up a dynamic catch target
+ SEPIA_DB_SUB: {
+        return &$DB::sub;
+    };
+    # we're dying!
+    last SEPIA_DB_SUB
+        if $DIE_LEVEL > 1 && defined $DIE_TO
+            && $DB::sub !~ /(?:^|::)\Q$DIE_TO\E$/;
+    undef $DIE_TO;
+    wantarray ? @DIE_RETURN : $DIE_RETURN[0]
+}
+# }
+
+sub repl_dbsub
+{
+    my $arg = shift;
+    if ($arg) {
+        *DB::sub = \&sub;
+    } else {
+        undef &DB::sub;
+    }
+}
+
 sub repl_lsbreak
 {
     no strict 'refs';
@@ -72,7 +118,7 @@ sub eval_in_env
         next unless /^([\$\@%])(.+)/;
         $str .= "local *$2 = \$Sepia::ENV->{'$_'}; ";
     }
-    eval "do { no strict; $str $expr }";
+    eval "do { no strict; package $Sepia::PACKAGE; $str $expr }";
 }
 
 sub tie_class
@@ -130,16 +176,13 @@ sub repl_inspect
 sub debug
 {
     my $new = Sepia::as_boolean(shift, $DB::trace);
-    return if $new == $DB::trace;
-    if ($new) {
-        # $^P = 0x2 | 0x10 | 0x100 | 0x200;
-        # *DB::DB = \&repl;
-        $DB::trace = 1;
-        print "debug ON\n";
+    print "debug ", $new ? "ON" : "OFF";
+    if ($new == $DB::trace) {
+        print " (unchanged)\n"
     } else {
-        $DB::trace = 0;
-        print "debug OFF\n";
+        print "\n";
     }
+    $DB::trace = $new;
 }
 
 sub breakpoint_file
@@ -176,7 +219,7 @@ sub repl_break
     return unless defined $f && defined $l;
     my $bp = breakpoint($f, $l, $cond);
     print "break $bp\n" if $bp;
- }
+}
 
 sub update_location
 {
@@ -205,6 +248,28 @@ sub repl_delete
     delete $h->{$l} if defined $h;
 }
 
+sub repl_finish
+{
+    # XXX: doesn't handle recursion, but oh, well...
+    my $sub = (caller $level + 4)[3];
+    if (exists $DB::sub{$sub}) {
+        my ($file, $start, $end) = $DB::sub{$sub} =~ /(.*):(\d+)-(\d+)/;
+        print STDERR "finish($sub): will stop at $file:$end\n";
+        # XXX: $end doesn't always work, since it may not have an
+        # executable statement on it.
+        breakpoint($file, $end-1, 'finish');
+        last repl;
+    } else {
+        print STDERR "yikes: @{[keys %DB::sub]}\n";
+    }
+}
+
+sub repl_toplevel
+{
+    local $STOPDIE;
+    die(bless [], __PACKAGE__);
+}
+
 sub add_repl_commands
 {
     define_shortcut 'delete', \&repl_delete,
@@ -213,15 +278,19 @@ sub add_repl_commands
         'debug [0|1]', 'Enable or disable debugging.';
     define_shortcut 'break', \&repl_break,
         'break [F:N [E]]',
-        'Set a breakpoint in F at line N (or at current position), enabled if E evalutes to true.';
+        'Break at file F, line N (or at current position) if E is true.';
     define_shortcut 'lsbreak', \&repl_lsbreak,
         'List breakpoints.';
+    # define_shortcut 'dbsub', \&repl_dbsub, '(Un)install DB::sub.';
     %Sepia::RK = abbrev keys %Sepia::REPL;
 }
 
 sub add_debug_repl_commands
 {
-
+    define_shortcut quit => \&repl_toplevel,
+        'quit', 'Quit the debugger, returning to the top level.';
+    define_shortcut toplevel => \&repl_toplevel,
+        'toplevel', 'Return to the top level.';
     define_shortcut up => sub {
         $level += shift || 1;
         update_location(4);
@@ -251,12 +320,17 @@ sub add_debug_repl_commands
         last repl;
     }, 'step [N]', 'Step N lines forward, entering subroutines.';
 
+    define_shortcut finish => \&repl_finish,
+        'finish', 'Finish the current subroutine.';
+
     define_shortcut list => \&repl_list,
         'list EXPR', 'List source lines of current file.';
     define_shortcut backtrace => \&repl_backtrace, 'show backtrace';
     define_shortcut inspect => \&repl_inspect,
         'inspect [N]', 'inspect lexicals in frame N (or current)';
     define_shortcut return => \&repl_return, 'return EXPR', 'return EXPR';
+    # define_shortcut xreturn => \&repl_xreturn, 'xreturn NAME EXPR',
+    #     'xreturn NAME EXPR';
     define_shortcut eval => \&repl_upeval,
         'eval EXPR', 'evaluate EXPR in current frame';      # DANGER!
 }
@@ -288,6 +362,11 @@ sub DB::DB
         my $cond = $main::{"_<$file"}{$line};
         if ($cond eq 'next') {
             delete $main::{"_<$file"}{$line};
+        } elsif ($cond eq 'finish') {
+            # remove temporary breakpoint and take one more step.
+            delete $main::{"_<$file"}{$line};
+            $DB::single = 1;
+            return;
         } else {
             return unless $Sepia::REPL{eval}->($cond);
         }
@@ -343,4 +422,24 @@ sub warn
     }
 }
 
+sub oops
+{
+    my $sig = shift;
+    if ($STOPDIE) {
+        my $trace = $DB::trace;
+        $DB::trace = 1;
+        local $level = 0;
+        local ($pack, $file, $line, $sub) = caller($level);
+        print "@_\n\tin $sub\nCaught signal $sig\n";
+        repl(
+        [die => sub { local $STOPDIE=0; CORE::die "Caught signal $sig; exiting." },
+         'Just die.'],
+        [quit => sub { local $STOPWARN=0; CORE::die "Caught signal $sig; exiting." },
+         'Just die.']);
+        $DB::trace = $trace;
+    } else {
+        Carp::confess "Caught signal $sig: continue at your own risk.";
+    }
+}
+
 1;
